"""
Module contains settings class for luna-admin
"""

from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    ConfiguratorAddressSettings as _ConfiguratorAddressSettings,
    ImageStoreAddressSettings as _ImageStoreAddressSettings,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsAdmin(BasePythonServiceSettingsClass):
    """
    Settings class for luna-admin
    Attributes:
        accountsAddress(ServiceAddressSettings): luna-accounts address settings
        lunaFacesAddress(ServiceAddressSettings): luna-faces address setting
        faceSamplesStorage(ImageStoreAddressSettings): face samples storage settings
        bodySamplesStorage(ImageStoreAddressSettings): body samples storage settings
        imageOriginStorage(ImageStoreAddressSettings): image origin storage settings
        lunaTaskResultStorage(ImageStoreAddressSettings): Luna Image Store address for task results
        servicesTimeouts(ServiceTimeoutsSettings): services timeouts settings
        lunaHandlersAddress(ServiceAddressSettings): luna-handlers address settings
        lunaEventsAddress(ServiceAddressSettings): luna-events address settings
        lunaSenderAddress (ServiceAddressSettings): luna-sender address settings
        lunaAPIAddress (ServiceAddressSettings): luna-api address settings
        lunaPythonMatcherAddress(ServiceAddressSettings): luna-python-matcher address settings
        lunaTasksAddress(ServiceAddressSettings): luna-tasks address settings
        lunaLicensesAddress(ServiceAddressSettings): luna-licenses address settings
        additionalServicesUsage(AdditionalServicesUsageSetting): support services container
        lunaMatcherProxyAddress(ServiceAddressSettings): luna-python-matcher address settings
        remoteSDKAddress(ServiceAddressSettings): luna-remote-sdk address settings
        lunaLambdaAddress(ServiceAddressSettings): luna-lambda address settings
        lunaVideoManagerAddress(ServiceAddressSettings): luna-video-manager address settings
    """

    serviceName = "luna-admin"

    def __init__(self):
        super().__init__()

        self.accountsAddress: Optional[_ServiceAddressSettings] = None
        self.lunaFacesAddress: Optional[_ServiceAddressSettings] = None
        self.faceSamplesStorage: Optional[_ImageStoreAddressSettings] = None
        self.bodySamplesStorage: Optional[_ImageStoreAddressSettings] = None
        self.imageOriginStorage: Optional[_ImageStoreAddressSettings] = None
        self.lunaTaskResultStorage: Optional[_ImageStoreAddressSettings] = None
        self.servicesTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.lunaHandlersAddress: Optional[_ServiceAddressSettings] = None
        self.lunaEventsAddress: Optional[_ServiceAddressSettings] = None
        self.lunaSenderAddress: Optional[_ServiceAddressSettings] = None
        self.lunaAPIAddress: Optional[_ServiceAddressSettings] = None
        self.lunaPythonMatcherAddress: Optional[_ServiceAddressSettings] = None
        self.lunaTasksAddress: Optional[_ServiceAddressSettings] = None
        self.lunaLicensesAddress: Optional[_ServiceAddressSettings] = None
        self.additionalServicesUsage: Optional[_AdditionalServicesUsageSetting] = None
        self.lunaMatcherProxyAddress: Optional[_ServiceAddressSettings] = None
        self.remoteSDKAddress: Optional[_ServiceAddressSettings] = None
        self.lunaLambdaAddress: Optional[_ServiceAddressSettings] = None
        self.lunaVideoManagerAddress: Optional[_ServiceAddressSettings] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()

        self.accountsAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_ACCOUNTS_ADDRESS", self["LUNA_ACCOUNTS_ADDRESS"]
        )
        self.lunaFacesAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_FACES_ADDRESS", self["LUNA_FACES_ADDRESS"]
        )
        self.lunaSenderAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_SENDER_ADDRESS", self["LUNA_SENDER_ADDRESS"]
        )
        self.lunaAPIAddress = _ServiceAddressSettings(self.serviceName, "LUNA_API_ADDRESS", self["LUNA_API_ADDRESS"])
        self.faceSamplesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS"],
        )
        self.bodySamplesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS"],
        )
        self.imageOriginStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_ADDRESS",
            self["LUNA_IMAGE_STORE_IMAGES_ADDRESS"],
        )
        self.lunaTaskResultStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_TASK_RESULT_ADDRESS",
            self["LUNA_IMAGE_STORE_TASK_RESULT_ADDRESS"],
        )
        self.lunaHandlersAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_HANDLERS_ADDRESS", self["LUNA_HANDLERS_ADDRESS"]
        )
        self.lunaEventsAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_EVENTS_ADDRESS", self["LUNA_EVENTS_ADDRESS"]
        )
        self.remoteSDKAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_REMOTE_SDK_ADDRESS", self["LUNA_REMOTE_SDK_ADDRESS"]
        )
        self.lunaTasksAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_TASKS_ADDRESS", self["LUNA_TASKS_ADDRESS"]
        )
        self.lunaLicensesAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LICENSES_ADDRESS", self["LUNA_LICENSES_ADDRESS"]
        )
        self.servicesTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_ADMIN_TIMEOUTS", self["LUNA_ADMIN_TIMEOUTS"]
        )
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )
        self.lunaPythonMatcherAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_ADDRESS",
            self["LUNA_PYTHON_MATCHER_ADDRESS"],
        )
        self.lunaMatcherProxyAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_MATCHER_PROXY_ADDRESS", self["LUNA_MATCHER_PROXY_ADDRESS"]
        )
        self.lunaLambdaAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LAMBDA_ADDRESS", self["LUNA_LAMBDA_ADDRESS"]
        )
        self.lunaVideoManagerAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_VIDEO_MANAGER_ADDRESS", self["LUNA_VIDEO_MANAGER_ADDRESS"]
        )
        return self

    @property
    def lunaConfiguratorAddress(self) -> _ConfiguratorAddressSettings:
        """Return configurator settings."""
        if self._configuratorAddress is None:
            raise RuntimeError("Configurator is disabled")
        return _ConfiguratorAddressSettings(self.serviceName, *self._configuratorAddress.rsplit("/", 1))
