"""
Module contains settings class for luna-api
"""

from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    ImageStoreAddressSettings as _ImageStoreAddressSettings,
    MetricsSettings as _MetricsSettings,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsApi(BasePythonServiceSettingsClass):
    """
    Settings class for luna-api
    Attributes:
        faceSamplesStorage(ImageStoreAddressSettings): face samples storage settings
        bodySamplesStorage(ImageStoreAddressSettings): body samples storage settings
        imageOriginStorage(ImageStoreAddressSettings): image origin storage settings
        objectsOriginStorage(ImageStoreAddressSettings): object origin storage settings
        faceSamplesStorageTimeouts(ServiceTimeoutsSettings): face samples storage timeouts settings
        bodySamplesStorageTimeouts(ServiceTimeoutsSettings): body samples storage timeouts settings
        imageOriginStorageTimeouts(ServiceTimeoutsSettings): image origin storage timeouts settings
        objectsOriginStorageTimeouts(ServiceTimeoutsSettings): objects origin storage timeouts settings
        facesAddress(ServiceAddressSettings): luna-faces address settings
        facesTimeouts(ServiceTimeoutsSettings): luna-faces timeouts settings
        eventsAddress(ServiceAddressSettings): luna-events address settings
        eventsTimeouts(ServiceTimeoutsSettings): luna-events timeouts settings
        tasksAddress(ServiceAddressSettings): luna-tasks address settings
        tasksTimeouts(ServiceTimeoutsSettings): luna-tasks timeouts settings
        pythonMatcherAddress(ServiceAddressSettings): luna-python-matcher address settings
        pythonMatcherTimeouts(ServiceTimeoutsSettings): luna-python-matcher timeouts settings
        senderAddress (ServiceAddressSettings): luna-sender address settings
        additionalServicesUsage(AdditionalServicesUsageSetting): support services container
        activePlugins(list): list of active plugins setting
        defaultFaceDescriptorVersion(int): default face descriptor version setting
        handlersAddress(ServiceAddressSettings): luna-handlers address settings
        handlersTimeouts(ServiceTimeoutsSettings): luna-handlers timeouts settings
        matcherProxyAddress(ServiceAddressSettings): luna-matcher-proxy address settings
        matcherProxyTimeouts(ServiceTimeoutsSettings): luna-matcher-proxy timeouts settings
        licensesAddress(ServiceAddressSettings): luna-licenses address settings
        accountsAddress(ServiceAddressSettings): luna-accounts address settings
        accountsTimeouts(ServiceTimeoutsSettings): luna-accounts timeouts settings
        remoteSDKAddress(_ServiceAddressSettings): external luna remote sdk address
        remoteSDKTimeouts(ServiceTimeoutsSettings): luna-remote-sdk timeouts settings
        allowLunaAccountAuthHeader(bool): whether to allow Luna-Account-Id auth header
        externalApiAddress (_ServiceAddressSettings): external luna api address
        remoteSDKAddress (_ServiceAddressSettings): external luna remote sdk address
        lambdaAddress (_ServiceAddressSettings): luna-lambda address settings
        lambdaTimeouts (_ServiceTimeoutsSettings): luna-lambda timeouts settings
        pluginsSettings (dict): luna api plugins settings
    """

    serviceName = "luna-api"

    def __init__(self):
        super().__init__()

        self.faceSamplesStorage: Optional[_ImageStoreAddressSettings] = None
        self.faceSamplesStorageTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.imageOriginStorage: Optional[_ImageStoreAddressSettings] = None
        self.objectsOriginStorage: Optional[_ImageStoreAddressSettings] = None

        self.bodySamplesStorage: Optional[_ImageStoreAddressSettings] = None
        self.bodySamplesStorageTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.imageOriginStorageTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.objectsOriginStorageTimeouts: Optional[_ServiceTimeoutsSettings] = None

        self.facesAddress: Optional[_ServiceAddressSettings] = None
        self.facesTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.eventsAddress: Optional[_ServiceAddressSettings] = None
        self.eventsTimeouts: Optional[_ServiceTimeoutsSettings] = None

        self.tasksAddress: Optional[_ServiceAddressSettings] = None
        self.tasksTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.pythonMatcherAddress: Optional[_ServiceAddressSettings] = None
        self.pythonMatcherTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.senderAddress: Optional[_ServiceAddressSettings] = None

        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )

        self.defaultFaceDescriptorVersion: Optional[int] = None
        self.handlersAddress: Optional[_ServiceAddressSettings] = None
        self.handlersTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.matcherProxyAddress: Optional[_ServiceAddressSettings] = None
        self.matcherProxyTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.licensesAddress: Optional[_ServiceAddressSettings] = None

        self.accountsAddress: Optional[_ServiceAddressSettings] = None
        self.accountsTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.remoteSDKAddress: Optional[_ServiceAddressSettings] = None
        self.remoteSDKTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.allowLunaAccountAuthHeader: Optional[bool] = None
        self.externalApiAddress: Optional[_ServiceAddressSettings] = None

        self.lambdaAddress: Optional[_ServiceAddressSettings] = None
        self.lambdaTimeouts: Optional[_ServiceTimeoutsSettings] = None

        self.videoManagerAddress: Optional[_ServiceAddressSettings] = None
        self.videoManagerTimeouts: Optional[_ServiceTimeoutsSettings] = None

        self.pluginsSettings: Optional[dict] = None
        self.metrics: Optional[_MetricsSettings] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.faceSamplesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS"],
        )
        self.faceSamplesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS"],
        )
        self.imageOriginStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_ADDRESS",
            self["LUNA_IMAGE_STORE_IMAGES_ADDRESS"],
        )
        self.objectsOriginStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_ADDRESS",
            self["LUNA_IMAGE_STORE_OBJECTS_ADDRESS"],
        )

        self.bodySamplesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS"],
        )
        self.bodySamplesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS"],
        )
        self.imageOriginStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_IMAGES_TIMEOUTS"],
        )
        self.objectsOriginStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_OBJECTS_TIMEOUTS",
            self["LUNA_IMAGE_STORE_OBJECTS_TIMEOUTS"],
        )

        self.facesAddress = _ServiceAddressSettings(self.serviceName, "LUNA_FACES_ADDRESS", self["LUNA_FACES_ADDRESS"])
        self.facesTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_FACES_TIMEOUTS",
            self["LUNA_FACES_TIMEOUTS"],
        )

        self.eventsAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_EVENTS_ADDRESS",
            self["LUNA_EVENTS_ADDRESS"],
        )
        self.remoteSDKAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_ADDRESS",
            self["LUNA_REMOTE_SDK_ADDRESS"],
        )
        self.remoteSDKTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_TIMEOUTS",
            self["LUNA_REMOTE_SDK_TIMEOUTS"],
        )
        self.eventsTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_EVENTS_TIMEOUTS",
            self["LUNA_EVENTS_TIMEOUTS"],
        )

        self.tasksAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_TASKS_ADDRESS",
            self["LUNA_TASKS_ADDRESS"],
        )
        self.tasksTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_TASKS_TIMEOUTS",
            self["LUNA_TASKS_TIMEOUTS"],
        )

        self.pythonMatcherAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_ADDRESS",
            self["LUNA_PYTHON_MATCHER_ADDRESS"],
        )
        self.pythonMatcherTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_TIMEOUTS",
            self["LUNA_PYTHON_MATCHER_TIMEOUTS"],
        )

        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )

        self.activePlugins = self["LUNA_API_ACTIVE_PLUGINS"]
        self.defaultFaceDescriptorVersion = self["DEFAULT_FACE_DESCRIPTOR_VERSION"]
        self.handlersAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_HANDLERS_ADDRESS", self["LUNA_HANDLERS_ADDRESS"]
        )
        self.handlersTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_HANDLERS_TIMEOUTS", self["LUNA_HANDLERS_TIMEOUTS"]
        )
        self.matcherProxyAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_MATCHER_PROXY_ADDRESS", self["LUNA_MATCHER_PROXY_ADDRESS"]
        )
        self.matcherProxyTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_PROXY_TIMEOUTS",
            self["LUNA_PYTHON_MATCHER_PROXY_TIMEOUTS"],
        )
        self.licensesAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LICENSES_ADDRESS", self["LUNA_LICENSES_ADDRESS"]
        )
        self.senderAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_SENDER_ADDRESS", self["LUNA_SENDER_ADDRESS"]
        )

        self.accountsAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_ACCOUNTS_ADDRESS", self["LUNA_ACCOUNTS_ADDRESS"]
        )
        self.accountsTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_ACCOUNTS_TIMEOUTS",
            self["LUNA_ACCOUNTS_TIMEOUTS"],
        )
        self.allowLunaAccountAuthHeader = self["ALLOW_LUNA_ACCOUNT_AUTH_HEADER"]
        self.externalApiAddress = _ServiceAddressSettings(
            self.serviceName,
            "EXTERNAL_LUNA_API_ADDRESS",
            self["EXTERNAL_LUNA_API_ADDRESS"],
        )

        self.lambdaAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LAMBDA_ADDRESS", self["LUNA_LAMBDA_ADDRESS"]
        )
        self.lambdaTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_LAMBDA_TIMEOUTS",
            self["LUNA_LAMBDA_TIMEOUTS"],
        )
        self.videoManagerAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_VIDEO_MANAGER_ADDRESS", self["LUNA_VIDEO_MANAGER_ADDRESS"]
        )
        self.videoManagerTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_VIDEO_MANAGER_TIMEOUTS",
            self["LUNA_VIDEO_MANAGER_TIMEOUTS"],
        )
        self.pluginsSettings = self["LUNA_API_PLUGINS_SETTINGS"]

        return self
