"""
Module contains settings class for luna-handlers
"""

from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    DBSetting as _DBSetting,
    DescriptorEncryption as _DescriptorEncryption,
    HandlersLimitsSettings as _HandlersLimitsSettings,
    ImageStoreAddressSettings as _ImageStoreAddressSettings,
    RedisDBSetting as _RedisDBSetting,
    RedisKeyStorageSetting as _RedisKeyStorageSetting,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsHandlers(BasePythonServiceSettingsClass):
    """
    Settings class for luna-handlers
    Attributes:
        db(DBSetting): database settings
        remoteSDKAddress(_ServiceAddressSettings): external luna remote sdk address
        remoteSDKTimeouts(ServiceTimeoutsSettings): luna-remote-sdk timeouts settings
        facesAddress(ServiceAddressSettings): luna-faces address settings
        facesTimeouts(ServiceTimeoutsSettings): luna-faces timeouts settings
        eventsAddress(ServiceAddressSettings): luna-events address settings
        eventsTimeouts(ServiceTimeoutsSettings): luna-events timeouts settings
        pythonMatcherAddress(ServiceAddressSettings): luna-python-matcher address settings
        pythonMatcherTimeouts(ServiceTimeoutsSettings): luna-python-matcher timeouts settings
        matcherProxyAddress(ServiceAddressSettings): luna-python-matcher address settings
        matcherProxyTimeouts(ServiceAddressSettings): luna-python-matcher address settings
        senderDB(RedisDBSetting): redis database settings
        additionalServicesUsage(AdditionalServicesUsageSetting): support services container
        storageTime(str): storage time setting
        defaultFaceDescriptorVersion(int): default face descriptor version setting
        defaultHumanDescriptorVersion(int): default human body descriptor version setting
        fetchImgTimeouts (ServiceTimeoutsSettings): timeouts for downloading images from external resources
        attributesStorageSettings (RedisStorageSetting): temporary attributes storage settings
        externalApiAddress (_ServiceAddressSettings): external luna api address
        lambdaUnitTimeouts (_ServiceTimeoutsSettings): luna-lambda timeouts settings
        descriptorEncryption(DescriptorEncryption): descriptors encryption settings
    """

    serviceName = "luna-handlers"

    def __init__(self):
        super().__init__()
        self.db: Optional[_DBSetting] = None
        self.remoteSDKAddress: Optional[_ServiceAddressSettings] = None
        self.remoteSDKTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.facesAddress: Optional[_ServiceAddressSettings] = None
        self.facesTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.eventsAddress: Optional[_ServiceAddressSettings] = None
        self.eventsTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.pythonMatcherAddress: Optional[_ServiceAddressSettings] = None
        self.pythonMatcherTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.matcherProxyAddress: Optional[_ServiceAddressSettings] = None
        self.matcherProxyTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.senderDB: Optional[_RedisDBSetting] = None
        self.additionalServicesUsage: Optional[_AdditionalServicesUsageSetting] = None
        self.storageTime: Optional[str] = None
        self.defaultFaceDescriptorVersion: Optional[int] = None
        self.defaultHumanDescriptorVersion: Optional[int] = None
        self.attributesStorageSettings: Optional[_RedisKeyStorageSetting] = None
        self.handlersLimitsSettings: Optional[_HandlersLimitsSettings] = None
        self.faceSamplesStorage: Optional[_ImageStoreAddressSettings] = None
        self.bodySamplesStorage: Optional[_ImageStoreAddressSettings] = None
        self.imageOriginStorage: Optional[_ImageStoreAddressSettings] = None
        self.faceSamplesStorageTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.bodySamplesStorageTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.imageOriginStorageTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.fetchImgTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.externalApiAddress: Optional[_ServiceAddressSettings] = None
        self.lambdaUnitTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.descriptorEncryption: Optional[_DescriptorEncryption] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.db = _DBSetting(self.serviceName, self["LUNA_HANDLERS_DB"])

        self.remoteSDKAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_ADDRESS",
            self["LUNA_REMOTE_SDK_ADDRESS"],
        )
        self.remoteSDKTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_TIMEOUTS",
            self["LUNA_REMOTE_SDK_TIMEOUTS"],
        )

        self.facesAddress = _ServiceAddressSettings(self.serviceName, "LUNA_FACES_ADDRESS", self["LUNA_FACES_ADDRESS"])
        self.facesTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_FACES_TIMEOUTS", self["LUNA_FACES_TIMEOUTS"]
        )

        self.eventsAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_EVENTS_ADDRESS",
            self["LUNA_EVENTS_ADDRESS"],
        )
        self.eventsTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_EVENTS_TIMEOUTS",
            self["LUNA_EVENTS_TIMEOUTS"],
        )

        self.pythonMatcherAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_ADDRESS",
            self["LUNA_PYTHON_MATCHER_ADDRESS"],
        )
        self.pythonMatcherTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_TIMEOUTS",
            self["LUNA_PYTHON_MATCHER_TIMEOUTS"],
        )

        self.matcherProxyAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_MATCHER_PROXY_ADDRESS", self["LUNA_MATCHER_PROXY_ADDRESS"]
        )
        self.matcherProxyTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_PYTHON_MATCHER_PROXY_TIMEOUTS", self["LUNA_PYTHON_MATCHER_PROXY_TIMEOUTS"]
        )

        self.senderDB = _RedisDBSetting(self.serviceName, self["REDIS_DB_ADDRESS"])

        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName,
            self["ADDITIONAL_SERVICES_USAGE"],
        )

        self.storageTime = self["storage_time"]

        self.defaultFaceDescriptorVersion = self["DEFAULT_FACE_DESCRIPTOR_VERSION"]
        self.defaultHumanDescriptorVersion = self["DEFAULT_HUMAN_DESCRIPTOR_VERSION"]

        self.attributesStorageSettings = _RedisKeyStorageSetting(
            self.serviceName, "ATTRIBUTES_STORAGE_POLICY", self["ATTRIBUTES_STORAGE_POLICY"]
        )

        self.faceSamplesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS"],
        )
        self.bodySamplesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS"],
        )
        self.imageOriginStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_ADDRESS",
            self["LUNA_IMAGE_STORE_IMAGES_ADDRESS"],
        )
        self.faceSamplesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS"],
        )
        self.bodySamplesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS"],
        )
        self.imageOriginStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_IMAGES_TIMEOUTS"],
        )
        self.fetchImgTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "FETCH_IMAGE_TIMEOUTS",
            self["FETCH_IMAGE_TIMEOUTS"],
        )
        self.externalApiAddress = _ServiceAddressSettings(
            self.serviceName,
            "EXTERNAL_LUNA_API_ADDRESS",
            self["EXTERNAL_LUNA_API_ADDRESS"],
        )
        self.handlersLimitsSettings = _HandlersLimitsSettings(self.serviceName, self["LUNA_HANDLERS_LIMITS"])
        self.lambdaUnitTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_LAMBDA_UNIT_TIMEOUTS",
            self["LUNA_LAMBDA_UNIT_TIMEOUTS"],
        )
        self.descriptorEncryption = _DescriptorEncryption(self.serviceName, self["DESCRIPTOR_ENCRYPTION"])

        return self
