"""
Module contains settings class for luna-matcher-proxy service
"""

from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    DBSetting as _DBSetting,
    DescriptorEncryption as _DescriptorEncryption,
    PlatformLimitsSettings as _PlatformLimitsSettings,
    RedisStorageSetting as _RedisStorageSetting,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsPythonMatcherProxy(BasePythonServiceSettingsClass):
    """
    Settings class for luna-matcher-proxy service.

    Attributes:
        lunaPythonMatcherAddress (ServiceAddressSettings): luna-python-matcher service address
        lunaPythonMatcherTimeouts (ServiceTimeoutsSettings): luna-python-matcher service timeouts
        defaultFaceDescriptorVersion (int): default face descriptor version setting
        facesDB (DBSetting): luna-faces db settings
        eventsDB (DBSetting): luna-events db settings
        additionalServicesUsage (AdditionalServicesUsageSetting): support services container
        storageTime (str): storage time setting
        attributesDB (RedisDBSetting): temporary attributes database settings
        platformLimits (PlatformLimitsSettings): platform limits
        descriptorEncryption(DescriptorEncryption): descriptors encryption settings
    """

    serviceName = "luna-matcher-proxy"

    def __init__(self):
        super().__init__()
        self.facesDB: Optional[_DBSetting] = None
        self.eventsDB: Optional[_DBSetting] = None
        self.attributesDB: Optional[_RedisStorageSetting] = None
        self.storageTime: Optional[str] = None
        self.additionalServicesUsage: Optional[_AdditionalServicesUsageSetting] = None
        self.lunaPythonMatcherAddress: Optional[_ServiceAddressSettings] = None
        self.lunaPythonMatcherTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.defaultFaceDescriptorVersion: Optional[int] = None
        self.defaultHumanDescriptorVersion: Optional[int] = None
        self.platformLimits: Optional[_PlatformLimitsSettings] = None
        self.descriptorEncryption: Optional[_DescriptorEncryption] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.facesDB = _DBSetting(self.serviceName, self["LUNA_FACES_DB"])
        self.eventsDB = _DBSetting(self.serviceName, self["LUNA_EVENTS_DB"])
        self.attributesDB = _RedisStorageSetting(self.serviceName, "LUNA_ATTRIBUTES_DB", self["LUNA_ATTRIBUTES_DB"])
        self.storageTime = self["STORAGE_TIME"]
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )
        self.lunaPythonMatcherAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_PYTHON_MATCHER_ADDRESS", self["LUNA_PYTHON_MATCHER_ADDRESS"]
        )
        self.lunaPythonMatcherTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_PROXY_TO_PYTHON_MATCHER_TIMEOUTS", self["LUNA_PROXY_TO_PYTHON_MATCHER_TIMEOUTS"]
        )
        self.defaultFaceDescriptorVersion = self["DEFAULT_FACE_DESCRIPTOR_VERSION"]
        self.defaultHumanDescriptorVersion = self["DEFAULT_HUMAN_DESCRIPTOR_VERSION"]
        self.platformLimits = _PlatformLimitsSettings(self.serviceName, self["PLATFORM_LIMITS"])
        self.descriptorEncryption = _DescriptorEncryption(self.serviceName, self["DESCRIPTOR_ENCRYPTION"])
