"""
Module contains settings class for luna-python-matcher service
"""

from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    DBSetting as _DBSetting,
    DescriptorCachingSetting as _DescriptorCachingSetting,
    DescriptorEncryption as _DescriptorEncryption,
    PlatformLimitsSettings as _PlatformLimitsSettings,
    RedisStorageSetting as _RedisStorageSetting,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsPythonMatcher(BasePythonServiceSettingsClass):
    """
    Settings class for luna-python-matcher service
    Attributes:
        facesDB (DBSetting): luna-faces db settings
        additionalServicesUsage (AdditionalServicesUsageSetting): support services container
        eventsDB (DBSetting): luna-events db settings
        storageTime (str): storage time setting
        defaultFaceDescriptorVersion (int): default face descriptor version setting
        attributesDB (RedisStorageSetting): temporary attributes database settings
        descriptorsCache (DescriptorCachingSetting): descriptor caching settings
        platformLimits (PlatformLimitsSettings): platform limits
        descriptorEncryption(DescriptorEncryption): descriptors encryption settings
    """

    serviceName = "luna-python-matcher"

    def __init__(self):
        super().__init__()
        self.facesDB: Optional[_DBSetting] = None
        self.eventsDB: Optional[_DBSetting] = None
        self.attributesDB: Optional[_RedisStorageSetting] = None
        self.storageTime: Optional[str] = None
        self.descriptorsCache: Optional[_DescriptorCachingSetting] = None
        self.additionalServicesUsage: Optional[_AdditionalServicesUsageSetting] = None
        self.defaultFaceDescriptorVersion: Optional[int] = None
        self.defaultHumanDescriptorVersion: Optional[int] = None
        self.platformLimits: Optional[_PlatformLimitsSettings] = None
        self.descriptorEncryption: Optional[_DescriptorEncryption] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.facesDB = _DBSetting(self.serviceName, self["LUNA_FACES_DB"])
        self.eventsDB = _DBSetting(self.serviceName, self["LUNA_EVENTS_DB"])
        self.attributesDB = _RedisStorageSetting(self.serviceName, "LUNA_ATTRIBUTES_DB", self["LUNA_ATTRIBUTES_DB"])
        self.storageTime = self["STORAGE_TIME"]
        self.descriptorsCache = _DescriptorCachingSetting(self.serviceName, self["DESCRIPTORS_CACHE"])
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )
        self.defaultFaceDescriptorVersion = self["DEFAULT_FACE_DESCRIPTOR_VERSION"]
        self.defaultHumanDescriptorVersion = self["DEFAULT_HUMAN_DESCRIPTOR_VERSION"]
        self.platformLimits = _PlatformLimitsSettings(self.serviceName, self["PLATFORM_LIMITS"])
        self.descriptorEncryption = _DescriptorEncryption(self.serviceName, self["DESCRIPTOR_ENCRYPTION"])
