import importlib
import os
from pathlib import Path
from time import perf_counter

from cow.errors.errors import Error
from cow.errors.exception import VLException
from cow.web.docs_handler import SpecDocumentsHandler
from cow.web.handlers import BaseHandler
from sanic import HTTPResponse

from ..private.exceptions import BaseLambdaException


class Handler(BaseHandler):
    """Lambda application handler"""

    async def post(self) -> HTTPResponse:
        main = importlib.import_module("lambda_main", "main").main
        try:
            result: dict = await main(self.request)
        except BaseLambdaException as exc:
            raise VLException(Error.LambdaError.format(exc.errorText), exc.statusCode, False)
        except Exception as exc:
            raise VLException(Error.LambdaError.format(str(exc)), 500, False)
        return self.success(201, outputJson=result)


class HealthCheckHandler(BaseHandler):
    """Lambda healthcheck handler"""

    async def get(self) -> HTTPResponse:
        start = perf_counter()
        outputJson = {"execution_time": perf_counter() - start}
        return self.success(200, outputJson=outputJson, extraHeaders={"Lambda-Type": self.request.appCtx.lambdaType})


class SpecDocHandler(SpecDocumentsHandler):
    """
    Handler for getting openapi documentations from static directory
    """

    def getDataFromStatic(self, extension: str) -> tuple[str, bytes]:
        """
        Get file body from a static directory with the specified extension

        Args:
            extension: extension for document file (html or yml)

        Returns:
            tuple (filename, document body)
        """
        try:
            STATIC_ROOT = Path(os.getcwd()) / "public" / self.app.ctx.lambdaType / "static"
        except Exception:  # local case
            STATIC_ROOT = Path(__file__).parent.parent / "public" / self.app.ctx.lambdaType / "static"
        if STATIC_ROOT.exists():
            for posixPath in STATIC_ROOT.iterdir():
                if "lambda" + extension == posixPath.name:
                    data = posixPath.read_bytes()
                    return posixPath.name, data
            raise VLException(Error.DocumentNotFoundError, 500, isCriticalError=False)
        raise VLException(Error.StaticNotFoundError, 500, isCriticalError=False)
