from argparse import ArgumentParser
from functools import partial
from socket import socket
from typing import Literal

from cow.manager.process import SanicProcess
from cow.manager.utils import createManager
from cow.web.configs_handler import ConfigsHandler

from .base_app import BaseLambdaApp as LambdaApp, LambdaTasksApp
from .configs.configs.services import SettingsLambdaTasksUnit, SettingsLambdaUnit
from .handlers import Handler, HealthCheckHandler, SpecDocHandler
from .helpers import getChecks4HandlersLambda, getChecks4StandaloneLambda, getChecks4TasksLambda, getParsedCMDArgs


def getCtxClass(lambdaType: Literal["standalone", "handlers", "tasks"]):
    """Get ctx class by lambda type"""
    try:
        from lambda_main import UserCtx
    except ImportError:

        class UserCtx:
            """Empty user application context"""

            ...

    return UserCtx


def getApp(lambdaType: Literal["standalone", "handlers", "tasks"], cmdArgs: ArgumentParser):
    """Get app class by lambda type"""
    userCtxCls = getCtxClass(lambdaType=lambdaType)
    lambdaName = cmdArgs.name
    if cmdArgs.name.startswith("lambda-"):
        lambdaName = cmdArgs.name[len("lambda-") :]
    if lambdaType == "tasks":
        app = LambdaTasksApp(
            serviceName=lambdaName, lambdaVersion=cmdArgs.version, lambdaType=lambdaType, userCtxCls=userCtxCls
        )
    else:
        app = LambdaApp(
            serviceName=lambdaName, lambdaVersion=cmdArgs.version, lambdaType=lambdaType, userCtxCls=userCtxCls
        )
    return app


def addRoute(application: LambdaApp | LambdaTasksApp):
    """Add route to app by lambda type"""
    if isinstance(application, LambdaApp):
        application.add_route(Handler.as_view(), "/main")
    application.add_route(HealthCheckHandler.as_view(), "/healthcheck")
    application.add_route(SpecDocHandler.as_view(), "/docs/spec")
    application.sanic.add_route(ConfigsHandler.as_view(), "/config", strict_slashes=True)
    return application


def getChecks(lambdaType: Literal["standalone", "handlers", "tasks"]):
    """Get checks by lambda type"""
    if lambdaType == "tasks":
        return getChecks4TasksLambda
    if lambdaType == "handlers":
        return getChecks4HandlersLambda
    return getChecks4StandaloneLambda


def getSettingsClass(lambdaType: Literal["standalone", "handlers", "tasks"]):
    """Get settings class by lambda type"""
    if lambdaType == "tasks":
        return SettingsLambdaTasksUnit
    return SettingsLambdaUnit


def _runLambda(sock: socket, lambdaType: Literal["standalone", "handlers", "tasks"]):
    """Run lambda"""
    cmdArgs = getParsedCMDArgs(lambdaType=lambdaType)
    application = getApp(lambdaType=lambdaType, cmdArgs=cmdArgs)
    application = addRoute(application=application)
    application.run(sock=sock)


def run(lambdaType: Literal["standalone", "handlers", "tasks"]):
    """Run lambda using process manager"""
    target = partial(_runLambda, lambdaType=lambdaType)
    procName = "lambda"
    processes = [SanicProcess(name=procName, target=target)]
    manager = createManager(
        processes,
        configClass=getSettingsClass(lambdaType=lambdaType),
        getChecks=getChecks(lambdaType=lambdaType),
        port=8000,
    )

    manager.run()
