import asyncio
from pathlib import Path
from time import time

import redis.asyncio as redis
from redis.asyncio.retry import Retry
from redis.backoff import ExponentialBackoff
from redis.exceptions import BusyLoadingError

from .structs import (
    QueueSubTask,
    QueueSubTasks,
    ResultAggregatorSubTask,
    SubTaskCancel,
    SubTaskError,
    SubTaskResult,
    SubTasksContentReply,
    SubTaskStart,
    TaskContentRequest,
    TaskResult,
)

projectName = Path(__file__).parent.parent.parent.stem

if projectName == "luna_tasks":
    from configs.config import TASK_SEPARATION_TIMEOUT
    from configs.configs.configs.settings.classes import RedisStorageSetting
    from crutches_on_wheels.cow.enums.tasks import TaskType
    from crutches_on_wheels.cow.errors.errors import Error
    from crutches_on_wheels.cow.errors.exception import VLException
else:
    from cow.enums.tasks import TaskType
    from cow.errors.errors import Error
    from cow.errors.exception import VLException

    from ...configs.configs.settings.classes import RedisStorageSetting

    TASK_SEPARATION_TIMEOUT = 60


class RedisNameGetter:
    """
    List of redis entities names
    """

    @staticmethod
    def taskContentSplitterQueue(taskType: TaskType, lambdaId: str | None = None):
        """Task content splitter redis sorted set name getter"""
        queue = f"luna_tasks_task2subtasks_get_{taskType.value}"
        if lambdaId is not None:
            queue += f"_{lambdaId}"
        return queue

    @staticmethod
    def setPrepareTaskContent(redisTaskId: str):
        """Task contents redis string name getter"""
        return f"luna_tasks_task2subtasks_{redisTaskId}"

    @staticmethod
    def subTaskResultList():
        """Subtask redis list name getter"""
        return "luna_tasks_subtasks_results"

    @staticmethod
    def subTaskStartList():
        """Subtask redis list name getter"""
        return "luna_tasks_subtasks_starts"

    @staticmethod
    def subTaskErrorList():
        """Subtask redis error list name getter"""
        return "luna_tasks_subtasks_errors"

    @staticmethod
    def taskResultList():
        """Task result redis list name getter"""
        return "luna_tasks_tasks_results"

    @staticmethod
    def subTasksQueue(taskType: TaskType, lambdaId: str | None = None):
        """Sub tasks redis sorted set name getter"""
        queue = f"luna_tasks_subtasks_{taskType.value}"
        if lambdaId is not None:
            queue += f"_{lambdaId}"
        return queue

    @staticmethod
    def resultAggregatorQueue(taskType: TaskType, lambdaId: str | None = None):
        """Result aggregator redis sorted set name getter"""
        queue = f"luna_tasks_tasks_aggregator_{taskType.value}"
        if lambdaId is not None:
            queue += f"_{lambdaId}"
        return queue

    @staticmethod
    def cancellerStream(taskType: TaskType, lambdaId: str | None = None):
        """Redis stream canceller stream name getter"""
        queue = f"luna_tasks_canceller_{taskType.value}"
        if lambdaId is not None:
            queue += f"_{lambdaId}"
        return queue

    @staticmethod
    def subTaskCancel():
        """Sub task on shutdown cancel redis list"""
        return "luna_tasks_sub_task_on_shutdown"


# types acceptable to push to redis
REDIS_REQUEST_TYPES = (
    TaskContentRequest
    | QueueSubTasks
    | ResultAggregatorSubTask
    | QueueSubTask
    | SubTasksContentReply
    | SubTaskResult
    | SubTaskError
    | TaskResult
    | SubTaskCancel
)


class _BaseRedisDBContext:
    """Base redis db context"""

    def __init__(self, redisSettings: RedisStorageSetting):
        self._redisSettings = redisSettings
        self._client = self._getClient(redisSettings=redisSettings)

    def reInitClient(self):
        """Re init redis client"""
        self._client = self._getClient(self._redisSettings)

    @staticmethod
    def _getClient(redisSettings: RedisStorageSetting):
        """Get redis client"""
        kwargs = {
            "retry": Retry(ExponentialBackoff(), 3),
            "retry_on_error": [BusyLoadingError, ConnectionError, TimeoutError],
            "password": redisSettings.password or None,
            "db": redisSettings.dbNumber,
        }
        if redisSettings.sentinel.sentinels:
            sentinel = redis.sentinel.Sentinel(sentinels=redisSettings.sentinel.sentinels, **kwargs)
            return sentinel.master_for(redisSettings.sentinel.masterName)
        else:
            return redis.Redis(host=redisSettings.host, port=redisSettings.port, **kwargs)

    async def close(self):
        """Close redis context"""
        await self._client.close()

    async def _removeFromCancellingStream(
        self,
        taskType: TaskType,
        subTaskId: int | bytes | None = None,
        taskId: int | bytes | None = None,
        lambdaId: str | None = None,
    ) -> bool:
        """
        Remove entity from canceller stream by task/subtask id
        Args:
            taskType: task type
            subTaskId: sub task id
            taskId: task id
        """
        redisName = RedisNameGetter.cancellerStream(taskType, lambdaId=lambdaId)
        keyOfInterest = subTaskId or taskId
        keyOfInterest = keyOfInterest if isinstance(keyOfInterest, bytes) else str(keyOfInterest).encode()
        if reply := (await self._client.xread({redisName: 0})):
            key = b"subtask" if subTaskId is not None else b"task"
            result = [row[0] for row in reply[0][1] if row[1].get(key, False) == keyOfInterest]
            if result:
                await self._client.xdel(redisName, result[0])
                return True
        return False

    async def _sendToRedis(self, request: REDIS_REQUEST_TYPES, lambdaId: str | None = None) -> None:
        """
        Send request to redis
        Args:
            request: request
        """
        if isinstance(request, TaskContentRequest):
            await self._client.zadd(
                RedisNameGetter.taskContentSplitterQueue(taskType=request.taskType, lambdaId=lambdaId),
                mapping={request.dump(): id(request.redisTaskId)},
            )
        elif isinstance(request, ResultAggregatorSubTask):
            await self._client.zadd(
                RedisNameGetter.resultAggregatorQueue(taskType=request.taskType, lambdaId=lambdaId),
                mapping={request.dump(): request.taskId},
            )
        elif isinstance(request, QueueSubTasks):
            await self._client.zadd(
                RedisNameGetter.subTasksQueue(request.taskType, lambdaId=lambdaId), mapping=request.dump()
            )
        elif isinstance(request, QueueSubTask):
            await self._client.zadd(
                RedisNameGetter.subTasksQueue(request.taskType, lambdaId=lambdaId),
                mapping={request.dump(): request.taskId},
            )
        elif isinstance(request, SubTasksContentReply):
            await self._client.set(
                RedisNameGetter.setPrepareTaskContent(request.redisTaskId),
                request.dump(),
                ex=TASK_SEPARATION_TIMEOUT,
            )
        elif isinstance(request, SubTaskResult):
            await self._client.lpush(RedisNameGetter.subTaskResultList(), request.dump())
        elif isinstance(request, SubTaskError):
            await self._client.lpush(RedisNameGetter.subTaskErrorList(), request.dump())
        elif isinstance(request, TaskResult):
            await self._client.lpush(RedisNameGetter.taskResultList(), request.dump())
        elif isinstance(request, SubTaskStart):
            await self._client.lpush(RedisNameGetter.subTaskStartList(), request.dump())
        elif isinstance(request, SubTaskCancel):
            await self._client.lpush(RedisNameGetter.subTaskCancel(), request.dump())
        else:
            raise NotImplementedError


class RouterRedisContext(_BaseRedisDBContext):
    """
    Tasks router redis context
    """

    async def sendToRedis(
        self, request: TaskContentRequest | QueueSubTasks | ResultAggregatorSubTask, lambdaId: str | None = None
    ):
        """
        Send request to redis
        Args:
            request: task content or sub task contents or result aggregator sub task
        """
        await self._sendToRedis(request, lambdaId=lambdaId)

    async def receiveSubTasksContent(
        self, redisTaskId: str, taskType: TaskType, lambdaId: str | None = None
    ) -> SubTasksContentReply:
        """
        Receive sub tasks contents
        Args:
            redisTaskId: redis task id
            taskType: task type

        Returns:
            sub tasks contents message
        """
        startTime = time()
        while time() - startTime < TASK_SEPARATION_TIMEOUT:
            if reply := (await self._client.get(RedisNameGetter.setPrepareTaskContent(redisTaskId))):
                return SubTasksContentReply.load(reply)
            await asyncio.sleep(0.1)
        id_ = float(id(redisTaskId))
        await self._client.zremrangebyscore(
            RedisNameGetter.taskContentSplitterQueue(taskType=taskType, lambdaId=lambdaId), min=id_, max=id_
        )
        raise VLException(
            Error.UnknownErrorLoopError.format(
                f"Failed to receive subtasks content in {TASK_SEPARATION_TIMEOUT} seconds"
            ),
            500,
            False,
        )

    async def cancelQueuedSubTasks(self, taskId: int, taskType: TaskType, lambdaId: str | None = None) -> None:
        """
        Cancel queued sub tasks
        Args:
            taskId: task id
            taskType: task type
        """
        await self._client.zremrangebyscore(
            RedisNameGetter.subTasksQueue(taskType, lambdaId=lambdaId), min=float(taskId), max=float(taskId)
        )

    async def cancelAggregatorSubTask(self, taskId: int, taskType: TaskType, lambdaId: str | None = None) -> None:
        """
        Cancel aggregator sub task
        Args:
            taskId: task id
            taskType: task type
        """
        await self._client.zremrangebyscore(
            RedisNameGetter.resultAggregatorQueue(taskType, lambdaId=lambdaId),
            min=float(taskId),
            max=float(taskId),
        )
        await self._client.xadd(RedisNameGetter.cancellerStream(taskType, lambdaId=lambdaId), {"task": taskId})

    async def cancelRunningSubTasks(
        self, subTaskIds: list[int], taskType: TaskType, lambdaId: str | None = None
    ) -> None:
        """
        Cancel running sub tasks - add them to redis canceller stream
        Args:
            subTaskIds: sub task ids
            taskType: task type
        """
        await asyncio.gather(
            *[
                self._client.xadd(RedisNameGetter.cancellerStream(taskType, lambdaId=lambdaId), {"subtask": subTaskId})
                for subTaskId in subTaskIds
            ]
        )

    async def getFinishedSubtasks(self):
        """
        Get finished sub tasks/sub tasks errors/task result.
        """
        if reply := (await self._client.rpop(RedisNameGetter.subTaskStartList())):
            return SubTaskStart.load(reply)
        if reply := (await self._client.rpop(RedisNameGetter.subTaskErrorList())):
            return SubTaskError.load(reply)
        if reply := (await self._client.rpop(RedisNameGetter.subTaskCancel())):
            return SubTaskCancel.load(reply)
        if reply := (await self._client.rpop(RedisNameGetter.subTaskResultList())):
            return SubTaskResult.load(reply)
        if reply := (await self._client.rpop(RedisNameGetter.taskResultList())):
            return TaskResult.load(reply)

    async def cleanCancellingStream(
        self,
        taskType: TaskType,
        subTaskId: int | None = None,
        taskId: int | None = None,
        lambdaId: str | None = None,
    ) -> None:
        """
        Remove entity from canceller stream
        Args:
            taskType: task type
            subTaskId: sub task id
            taskId: task id
        """
        await self._removeFromCancellingStream(taskType=taskType, subTaskId=subTaskId, taskId=taskId, lambdaId=lambdaId)


class WorkerRedisContext(_BaseRedisDBContext):
    """
    Worker redis context
    """

    async def getSeparationRequest(self, taskType: TaskType, lambdaId: str | None = None) -> TaskContentRequest | None:
        """
        Get task content separation request
        Args:
            taskType: task type
        """
        if reply := (
            await self._client.zpopmin(
                RedisNameGetter.taskContentSplitterQueue(taskType=taskType, lambdaId=lambdaId), count=1
            )
        ):
            return TaskContentRequest.load(reply[0][0])

    async def sendToRedis(self, request: REDIS_REQUEST_TYPES, lambdaId: str | None = None):
        """Send request to redis"""
        await self._sendToRedis(request, lambdaId=lambdaId)

    async def getSubTask2Process(self, taskType: TaskType, lambdaId: str | None = None) -> QueueSubTask:
        """Get sub task to process"""
        if reply := (
            await self._client.zpopmin(RedisNameGetter.subTasksQueue(taskType=taskType, lambdaId=lambdaId), count=1)
        ):
            return QueueSubTask.load(reply[0][0])

    async def getResultAggregatorSubTask(self, taskType: TaskType, lambdaId: str | None = None):
        """Get result aggregator sub task"""
        if reply := (
            await self._client.zpopmin(
                RedisNameGetter.resultAggregatorQueue(taskType=taskType, lambdaId=lambdaId), count=1
            )
        ):
            return ResultAggregatorSubTask.load(reply[0][0])

    async def isNeedToCancel(
        self, taskType: TaskType, subTaskId: bytes | None = None, taskId: bytes | None = None
    ) -> bool:
        """Check whether task/subtask is need to be cancelled"""
        return await self._removeFromCancellingStream(taskType=taskType, taskId=taskId, subTaskId=subTaskId)
